unit TV_DragDrop_U1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls,  TreeUtils;

const
   IMG_NODE_ROOT = 0;
   IMG_NODE_FILE_CLOSED = 1;
   IMG_NODE_FILE_OPEN = 2;
   IMG_NODE_FOLDER_CLOSED = 3;
   IMG_NODE_FOLDER_OPEN = 4;


type
      {Enum used for easily identifying nodes}
   eNodeType = (ntUnknown,  ntRoot,  ntFile,  ntFolder);

  TForm1 = class(TForm)
    tv_eg5: TTreeView;
    but_AddFolder: TButton;
    but_Remove: TButton;
    ImageList1: TImageList;
    but_AddFile: TButton;
    procedure but_RemoveClick(Sender: TObject);
    procedure but_AddFolderClick(Sender: TObject);
    procedure but_AddFileClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure tv_eg5DragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure tv_eg5DragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
  private
     function GetNodeType(  Node : TTreeNode  ) : eNodeType;
     procedure AddRootNode;
     procedure AddNode(  NodeType : eNodeType  );
     function IsNodeAllowed(  ParentNode : TTreeNode;
                              NewNodesType : eNodeType
                            ) : boolean;
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}


procedure TForm1.FormCreate(Sender: TObject);
begin
      {Add the root node}
   AddRootNode;
end;



//////////////////////////////////////////////////////////////
// Returns one of the eNodeType values to indicate what type
//  of node param 1 is.
//////////////////////////////////////////////////////////////
function TForm1.GetNodeType(  Node : TTreeNode  ) : eNodeType;
begin
   if(  Node = nil  ) then
   begin
      Result := ntUnknown;
      Exit;
   end;

      {Determine what type of node this is by looking at the
        node's ImageIndex}
   case Node.ImageIndex of
      IMG_NODE_ROOT : Result := ntRoot;
      IMG_NODE_FILE_CLOSED : Result := ntFile;
      IMG_NODE_FOLDER_CLOSED : Result := ntFolder;
   else
         {Node should be one of the above...}
      Result := ntUnknown;
   end;
end;



///////////////////////////////////////////////////////
// Check if a new node of type NewNodeType may be
//   created as a child off ParentNode
///////////////////////////////////////////////////////
function TForm1.IsNodeAllowed(  ParentNode : TTreeNode;
                                NewNodesType : eNodeType
                              ) : boolean;
begin
   case GetNodeType(  ParentNode  ) of
      ntRoot :
      begin
            {A root may contain any type of node}
         Result := true;
      end;

      ntFolder :
      begin
            {Folder may contain any type of node}
         Result := true;
      end;

      ntFile :
      begin;
            {Files may have no sub-items}
         Result := false;
      end;
   else
         {Unknown node type, dont allow any operations}
      Result := false;
   end;
end;



///////////////////////////////////////////////////
// Procedure used to add a file / folder node
///////////////////////////////////////////////////
procedure TForm1.AddNode(  NodeType : eNodeType  );
var
   sText : string;
begin
      {If nothing is selected}
   if(  tv_eg5.Selected = nil  ) then
   begin
         {There is a root, so user must first select a node}
      MessageBeep(  -1  );
      ShowMessage(  'Select parent node'  );
      Exit;
   end
   else begin
         {Get a name for the new node}
      if(   not InputQuery(  'New Node',  'Caption ?',  sText  )   ) then
         Exit;

         {Check if this name is already in use}
      if(   IsDuplicateName(  tv_eg5.Selected.GetFirstChild,
                              sText,
                              true
                            )
         ) then
      begin
         MessageBeep(  -1  );
         ShowMessage(  'A node with this name already exists'  );
         Exit;
      end;

         {Check if adding this type of node is allowed}
      if(   not IsNodeAllowed(  tv_eg5.Selected,  NodeType  )   ) then
      begin
         MessageBeep(  -1  );
         ShowMessage(  'Cant creat this type of node here'  );
         Exit;
      end;

         {Add the node}
      with tv_eg5.Items.AddChildFirst(  tv_eg5.Selected,  sText  ) do
      begin
         case NodeType of
            ntFolder :
            begin
                  {Set the image used when the node is not selected}
               ImageIndex := IMG_NODE_FOLDER_CLOSED;
                  {Image used when the node is selected}
               SelectedIndex := IMG_NODE_FOLDER_OPEN;
               MakeVisible;
            end;

            ntFile :
            begin
                  {Set the image used when the node is not selected}
               ImageIndex := IMG_NODE_FILE_CLOSED;
                  {Image used when the node is selected}
               SelectedIndex := IMG_NODE_FILE_OPEN;
               MakeVisible;
            end;
         else
               {Trying to add a node that is not a file or a folder,
                 this is not allowed. So remove the node that was
                 just created.}
            Delete;
         end;
      end;
   end;
end;



///////////////////////////////////
// Adds the root to an empty tree
///////////////////////////////////
procedure TForm1.AddRootNode;
begin
      {If the tree is empty}
   if(  tv_eg5.Items.Count = 0  ) then
   begin
         {Add the root node}
      with tv_eg5.Items.AddFirst(  nil,  'Root'  ) do
      begin
         Selected := true;

            {Set the roots image index}
         ImageIndex := IMG_NODE_ROOT;
            {Set the roots selected index. The same image is uses
               as for the ImageIndex}
         SelectedIndex := IMG_NODE_ROOT;
      end;
   end
end;



//////////////////////////////////////////////////
// User wants to remove the selected item
//////////////////////////////////////////////////
procedure TForm1.but_RemoveClick(Sender: TObject);
begin
      {Make sure somthing is selected, before trying to
        delete it}
   if(  tv_eg5.Selected = nil  ) then
   begin
      MessageBeep(  -1  );
      ShowMessage(  'Nothing selected'  );
      Exit;
   end;

      {Dont allow user to delete the root node}
   if(  tv_eg5.Selected.Level = 0  ) then
   begin
      MessageBeep(  -1  );
      ShowMessage(  'Cant delete the root node'  );
      Exit;
   end;

      {Delete the node}
   tv_eg5.Selected.Delete;
end;



procedure TForm1.but_AddFolderClick(Sender: TObject);
begin
   AddNode(  ntFolder  );
end;



procedure TForm1.but_AddFileClick(Sender: TObject);
begin
   AddNode(  ntFile  );
end;



procedure TForm1.tv_eg5DragDrop(Sender, Source: TObject; X, Y: Integer);
var
   TargetNode : TTreeNode;
   SourceNode : TTreeNode;
begin
/////////////////////////////////////////
// Somthing has just been droped
/////////////////////////////////////////


   with tv_eg5 do
   begin
         {Get the node the item was dropped on}
      TargetNode := GetNodeAt(  X,  Y  );
         {Just to make things a bit easier}
      SourceNode := Selected;


         {Make sure somthing was droped onto}
      if(  TargetNode = nil  ) then
      begin
         EndDrag(  false  );
         Exit;
      end;

         {Dropping onto self or onto parent?}
      if(  (TargetNode = Selected) or
           (TargetNode = Selected.Parent)
        ) then
      begin
         MessageBeep(  MB_ICONEXCLAMATION  );
         ShowMessage(  'Destination node is the same as the source node'  );
         EndDrag(  false  );
         Exit;
      end;

         {No drag-drop of the root allowed}
      if(  SourceNode.Level = 0  ) then
      begin
         MessageBeep(  MB_ICONEXCLAMATION  );
         ShowMessage(  'Cant drag/drop the root'  );
         EndDrag(  false  );
         Exit;
      end;


         {Can't drop a parent onto a child}
      if(   IsAParentNode(  Selected,  TargetNode  )   ) then
      begin
         MessageBeep(  MB_ICONEXCLAMATION  );
         ShowMessage(  'Cant drop parent onto child'  );
         EndDrag(  false  );
         Exit;
      end;

         {Does a node with this name exists as a child of TargetNde}
      if(   IsDuplicateName(  TargetNode.GetFirstChild,  SourceNode.Text,  true  )   ) then
      begin
         MessageBeep(  MB_ICONEXCLAMATION  );
         ShowMessage(  'A node with this name already exists'  );
         EndDrag(  false  );
         Exit;
      end;

      //////////////////////////////////////////////////////////////
      // Nothing differant up to here.  Just the normal drag and
      //   drop checking.  Now the code to make sure that enforce
      //   "the rules".  Eg books may contain no sub-nodes
      //////////////////////////////////////////////////////////////

         {Use the IsNodeAllowed function to test if the node
           may be dropped here}
      if(  not IsNodeAllowed(   TargetNode,
                                GetNodeType(  SourceNode  )
                              )
         ) then
      begin
         MessageBeep(  -1  );
         ShowMessage(  'You cant drop this type of node here!'  );
         EndDrag(  false  );
         Exit;
      end;


         {Drag drop was valid so move the nodes}
      MoveTreeNode(  tv_eg5,  SourceNode,  TargetNode  );

         {Delete the old node}
      SourceNode.Delete;


         {Show the nodes that were just moved}
      TargetNode.Expand(  true  );
   end;
end;



procedure TForm1.tv_eg5DragOver(Sender, Source: TObject; X, Y: Integer;  State: TDragState; var Accept: Boolean);
begin
/////////////////////////////////////////
// Decide if drag-drop is to be allowed
/////////////////////////////////////////

   Accept := false;

      {Only accept drag and drop from a TTreeView}
   if(  Sender is TTreeView  ) then
         {Only accept from self}
      if(  TTreeView(Sender) = tv_eg5  ) then
         Accept := true;
end;


end.
