unit TV_ListView_U1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls,  TreeUtils,  CommCtrl;

const
   IMG_NODE_ROOT = 0;
   IMG_NODE_FILE_CLOSED = 1;
   IMG_NODE_FILE_OPEN = 2;
   IMG_NODE_FOLDER_CLOSED = 3;
   IMG_NODE_FOLDER_OPEN = 4;
   IMG_TO_PARENT = 5;


type
      {Enum used for easily identifying nodes}
   eNodeType = (ntUnknown,  ntRoot,  ntFile,  ntFolder);

  TForm1 = class(TForm)
    tv_eg5: TTreeView;
    but_AddFolder: TButton;
    but_Remove: TButton;
    ImageList1: TImageList;
    but_AddFile: TButton;
    lv_Link: TListView;
    procedure but_RemoveClick(Sender: TObject);
    procedure but_AddFolderClick(Sender: TObject);
    procedure but_AddFileClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure tv_eg5Change(Sender: TObject; Node: TTreeNode);
    procedure lv_LinkDblClick(Sender: TObject);
    procedure lv_LinkEdited(Sender: TObject; Item: TListItem;
      var S: String);
    procedure lv_LinkEditing(Sender: TObject; Item: TListItem;
      var AllowEdit: Boolean);
    procedure tv_eg5Editing(Sender: TObject; Node: TTreeNode;
      var AllowEdit: Boolean);
  private
     function GetNodeType(  Node : TTreeNode  ) : eNodeType;
     procedure AddRootNode;
     procedure AddNode(  NodeType : eNodeType  );
     procedure UpdateListView;

     function IsNodeAllowed(  ParentNode : TTreeNode;
                              NewNodesType : eNodeType
                            ) : boolean;
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}


procedure TForm1.FormCreate(Sender: TObject);
begin
      {Add the root node}
   AddRootNode;
end;



//////////////////////////////////////////////////////////////
// Returns one of the eNodeType values to indicate what type
//  of node param 1 is.
//////////////////////////////////////////////////////////////
function TForm1.GetNodeType(  Node : TTreeNode  ) : eNodeType;
begin
   if(  Node = nil  ) then
   begin
      Result := ntUnknown;
      Exit;
   end;

      {Determine what type of node this is by looking at the
        node's ImageIndex}
   case Node.ImageIndex of
      IMG_NODE_ROOT : Result := ntRoot;
      IMG_NODE_FILE_CLOSED : Result := ntFile;
      IMG_NODE_FOLDER_CLOSED : Result := ntFolder;
   else
         {Node should be one of the above...}
      Result := ntUnknown;
   end;
end;



///////////////////////////////////////////////////////
// Check if a new node of type NewNodeType may be
//   created as a child off ParentNode
///////////////////////////////////////////////////////
function TForm1.IsNodeAllowed(  ParentNode : TTreeNode;
                                NewNodesType : eNodeType
                              ) : boolean;
begin
   case GetNodeType(  ParentNode  ) of
      ntRoot :
      begin
            {A root may contain any type of node}
         Result := true;
      end;

      ntFolder :
      begin
            {Folder may contain any type of node}
         Result := true;
      end;

      ntFile :
      begin;
            {Files may have no sub-items}
         Result := false;
      end;
   else
         {Unknown node type, dont allow any operations}
      Result := false;
   end;
end;



///////////////////////////////////////////////////
// Procedure used to add a file / folder node
///////////////////////////////////////////////////
procedure TForm1.AddNode(  NodeType : eNodeType  );
var
   sText : string;
begin
      {If nothing is selected}
   if(  tv_eg5.Selected = nil  ) then
   begin
         {There is a root, so user must first select a node}
      MessageBeep(  -1  );
      ShowMessage(  'Select parent node'  );
      Exit;
   end
   else begin
         {Get a name for the new node}
      if(   not InputQuery(  'New Node',  'Caption ?',  sText  )   ) then
         Exit;

         {Check if this name is already in use}
      if(   IsDuplicateName(  tv_eg5.Selected.GetFirstChild,
                              sText,
                              true
                            )
         ) then
      begin
         MessageBeep(  -1  );
         ShowMessage(  'A node with this name already exists'  );
         Exit;
      end;

         {Check if adding this type of node is allowed}
      if(   not IsNodeAllowed(  tv_eg5.Selected,  NodeType  )   ) then
      begin
         MessageBeep(  -1  );
         ShowMessage(  'Cant creat this type of node here'  );
         Exit;
      end;

         {Add the node}
      with tv_eg5.Items.AddChildFirst(  tv_eg5.Selected,  sText  ) do
      begin
         case NodeType of
            ntFolder :
            begin
                  {Set the image used when the node is not selected}
               ImageIndex := IMG_NODE_FOLDER_CLOSED;
                  {Image used when the node is selected}
               SelectedIndex := IMG_NODE_FOLDER_OPEN;
               MakeVisible;
            end;

            ntFile :
            begin
                  {Set the image used when the node is not selected}
               ImageIndex := IMG_NODE_FILE_CLOSED;
                  {Image used when the node is selected}
               SelectedIndex := IMG_NODE_FILE_OPEN;
               MakeVisible;
            end;
         else
               {Trying to add a node that is not a file or a folder,
                 this is not allowed. So remove the node that was
                 just created.}
            Delete;
         end;
      end;
   end;

      {Update}
   UpdateListView;
end;



///////////////////////////////////
// Adds the root to an empty tree
///////////////////////////////////
procedure TForm1.AddRootNode;
begin
      {If the tree is empty}
   if(  tv_eg5.Items.Count = 0  ) then
   begin
         {Add the root node}
      with tv_eg5.Items.AddFirst(  nil,  'Root'  ) do
      begin
         Selected := true;

            {Set the roots image index}
         ImageIndex := IMG_NODE_ROOT;
            {Set the roots selected index. The same image is uses
               as for the ImageIndex}
         SelectedIndex := IMG_NODE_ROOT;
      end;
   end
end;



//////////////////////////////////////////////////
// User wants to remove the selected item
//////////////////////////////////////////////////
procedure TForm1.but_RemoveClick(Sender: TObject);
begin
      {Make sure somthing is selected, before trying to
        delete it}
   if(  tv_eg5.Selected = nil  ) then
   begin
      MessageBeep(  -1  );
      ShowMessage(  'Nothing selected'  );
      Exit;
   end;

      {Dont allow user to delete the root node}
   if(  tv_eg5.Selected.Level = 0  ) then
   begin
      MessageBeep(  -1  );
      ShowMessage(  'Cant delete the root node'  );
      Exit;
   end;

      {Delete the node}
   tv_eg5.Selected.Delete;

      {Update}
   UpdateListView;
end;



procedure TForm1.but_AddFolderClick(Sender: TObject);
begin
   AddNode(  ntFolder  );
end;



procedure TForm1.but_AddFileClick(Sender: TObject);
begin
   AddNode(  ntFile  );
end;



procedure TForm1.UpdateListView;
var
   ChildNode : TTreeNode;
begin
   lv_Link.Items.Clear;

      {If nothing is selected}
   if(  tv_eg5.Selected = nil  ) then
      Exit;


      {Does this node have a parent?}
   if(  tv_eg5.Selected.Parent <> nil  ) then
   begin
        {Add the "up to parent" node}
      with lv_Link.Items.Add do
      begin
         Caption := 'To Parent';
            {Set the image}
         ImageIndex := IMG_TO_PARENT;

            {Node that ListItem is linking to (ie Parent Node)}
         Data := tv_eg5.Selected.Parent.ItemId;
      end;
   end;


   ////////////////////////////////////////////////
   // Now get all the selected node's child nodes
   ////////////////////////////////////////////////
   lv_Link.Items.BeginUpdate;

      {Get first child}
   ChildNode := tv_eg5.Selected.GetFirstChild;

   while(  ChildNode <> nil  ) do
   begin
        {Add the "up to parent" node}
      with lv_Link.Items.Add do
      begin
         Caption := ChildNode.Text;
            {Set the image}
         ImageIndex := ChildNode.ImageIndex;

            {Node that ListItem is linking to}
         Data := ChildNode.ItemId;
      end;

         {Next child node}
      ChildNode := tv_eg5.Selected.GetNextChild(  ChildNode  );
   end;

   lv_Link.Items.EndUpdate;   
end;



procedure TForm1.tv_eg5Change(Sender: TObject; Node: TTreeNode);
begin
   UpdateListView;
end;



procedure TForm1.lv_LinkDblClick(Sender: TObject);
var
   LinkNode : TTreeNode;
begin
      {Make sure there is a selection}
   if(  lv_Link.Selected = nil  ) then
      Exit;

      {Find the node in the TreeView that corresponds to
        this ListView Item}
   LinkNode := tv_eg5.Items.GetNode(  HTreeItem(lv_Link.Selected.Data)  );

      {If the node was found, select it}
   if(  LinkNode <> nil  ) then
      LinkNode.Selected := true;
end;



procedure TForm1.lv_LinkEditing(Sender: TObject; Item: TListItem;  var AllowEdit: Boolean);
begin
      {Can edit all but the "To Parent" item}
   AllowEdit := (Item.ImageIndex <> IMG_TO_PARENT);
end;



procedure TForm1.lv_LinkEdited(Sender: TObject; Item: TListItem;  var S: String);
var
   LinkNode : TTreeNode;
begin
      {Find the node that is being renamed}
   LinkNode := tv_eg5.Items.GetNode(  HTreeItem(Item.Data)  );

      {If the node was found}
   if(  LinkNode <> nil  ) then
   begin
         {Does another node already have this name?}
      if(   IsDuplicateName(  LinkNode,  s,  false  )   ) then
      begin
         MessageBeep(  -1  );
         ShowMessage(  'Duplicate Name!'  );
         
            {Revert to original name}
         S := Item.Caption;
      end
      else begin
            {Name is valid so allow rename}
         LinkNode.Text := s;
      end;
   end
   else begin
      {For some reason the link was not found, give ListIem its
        old name}
      S := Item.Caption;
   end;
end;



procedure TForm1.tv_eg5Editing(Sender: TObject; Node: TTreeNode; var AllowEdit: Boolean);
begin
      {All but root may be renamed}
   AllowEdit := (Node.ImageIndex <> IMG_NODE_ROOT);
end;

end.
